/**************************************************************************
 *                       
 *  Name:        bosch.c  
 *  Description: Luftdruckberechnung mit BMP-Sensor von Bosch   
 *  Date  :      19.05.2010 
 *  Author:      
 *  Revision List:
  --------------------------------------------------------------------------
  Date     | Author          | Change
  --------------------------------------------------------------------------
  19.05.10 |                 | Ersterstellung
  --------------------------------------------------------------------------
           |                 |
  --------------------------------------------------------------------------
 **************************************************************************/

/***************************************************************************
  HEADER-FILES (Only those that are needed in this file)
****************************************************************************/

#include "bosch.h"
//#include "maja.h"

/* Foreign headerfiles */
#include "i2c.h"


/***************************************************************************
*    Konstanten
****************************************************************************/
const byte abRgnb[] = 
  /*  AC1   AC2   AC3   AC4   AC5   AC6   B1    B2    MB    MC    MD */  
    {0xAA, 0xAC, 0xAE, 0xB0, 0xB2, 0xB4, 0xB6, 0xB8, 0xBA, 0xBC, 0xBE};

enum {
    enBpReadCalibPrep = 0,    /* Kalibrierungsdaten lesen */
    enBpReadCalibWait,
    enBpTempPrep,
    enBpTempRead,
    enBpBaroPrep,
    enBpBaroRead,
    enBpCalculate,
    enBpError
    };

/***************************************************************************
*    Variablen
****************************************************************************/
struct stBpCalibrate {   /* Kalibrierungsdaten */
    int16 AC1;
    int16 AC2;
    int16 AC3;
    word  AC4;
    word  AC5;
    word  AC6;
    int16 B1;
    int16 B2;
    int16 MB;
    int16 MC;
    int16 MD;
    } stBpCalibrate; 

byte bBpSize; /*
* Description : Variable fr Kalibrierungsdaten 
*/

byte abBpData[4]; /*
* Description : Sensordaten 
*/

byte bBpFlag; /*
* Description : Zustandsflag
*/

word *pBpCalib; /*
* Description : Zeiger auf Kalibrierungsdaten 
*/

I2C_stTrx stTrxRW; /*
* Description : Speicherstruktur fr I2C Zugriff
*/

byte bBpState; /*
* Description : Zustandsautomat fr Drucksensor
*/

dword dwUT; /*
* Description : Rohdatum der Temperatur
*/

int32 iTemperature; /*
* Description : Temperatur
*/

dword dwUP; /*
* Description : Rohdatum des Druckes
*/

int32 iPressure; /*
* Description : Luftdruck
*/

int32 iDeltaTemp; 

int32 x1, x2, b5, b6, x3, b3, p;
dword b4, b7;

I2C_stTrx stBpInitList[8];

I2C_stTrx *pBpList;



/***************************************************************************
*    Funktion: Initialisierung
*    Autor:    
****************************************************************************/
void BP_vInit( void ){
    bBpSize = 0;
    bBpState = enBpReadCalibPrep;
    pBpCalib = (word*)&stBpCalibrate;
    stTrxRW.bCycl = 0;              /* einmalige Abfrage */
    stTrxRW.bTime = 0;
    stTrxRW.bType = enI2C_Read;     /* Lesen */
    stTrxRW.bAddr = BP_nSensorAddr; /* Sensoradresse */
    stTrxRW.bSize = sizeof(word);   /* Anzahl der Datenbytes */
    stTrxRW.pData = &abBpData[0];   /* Datenspeicher */
    stTrxRW.pFlag = &bBpFlag;       /* Zustand dieses Speichers */
    iPressure = -10000;
    }

/***************************************************************************
*    Funktion: Anzeigezustand
*    Autor:    
****************************************************************************/
void BP_vMain( cbyte cID ){
    int32 i;
    switch(bBpState){
        case enBpReadCalibPrep:
            stTrxRW.bRgnb = abRgnb[bBpSize];      /* gewnschtes Register */
            bBpFlag = enI2C_Clear;                /* Flag lschen */
            I2C_bTrxRequest(stTrxRW);             /* Anforderung anmelden */
            bBpState = enBpReadCalibWait;
            break;
        case enBpReadCalibWait:
            if(bBpFlag == enI2C_Ready){           /* Daten vorhanden */
                *pBpCalib  = (word)(abBpData[0] << 8);
                *pBpCalib |= abBpData[1];
                if(bBpSize < sizeof(abRgnb)){     /* nchsten Kalibrierungswert auslesen */
                    pBpCalib++;
                    bBpSize++;
                    bBpState = enBpReadCalibPrep;
                    }
                else{
                    bBpState = enBpTempPrep;
                    }
                }
            break;
        case enBpTempPrep:                        /* Vorbereitung Temperatur auslesen */
            stTrxRW.bType = enI2C_Write;          /* Schreibanforderung */
            stTrxRW.bSize = 1;                    /* nur ein Byte mglich */
            stTrxRW.bRgnb = BP_nSensorRgnb;       /* Datenregisternummer */  
            abBpData[0] = BP_nTempValue;          /* Schreibdaten */
            bBpFlag = enI2C_Clear;
            I2C_bTrxRequest(stTrxRW); 
            bBpState = enBpTempRead;
            break;
        case enBpTempRead:                        /* Temperatur-Roh-Daten abfragen */
            if(bBpFlag == enI2C_Ready){           /* Anforderung erledigt */
                stTrxRW.bType = enI2C_Read;       /* Werte auslesen */
                stTrxRW.bSize = sizeof(word);     /* 16 Bit */
                stTrxRW.bRgnb = BP_nBTValue;      /* Datenregisternummer */  
                bBpFlag = enI2C_Clear;
                I2C_bTrxRequest(stTrxRW); 
                bBpState = enBpBaroPrep;
                }
            break;
        case enBpBaroPrep:                        /* Temp-Rohwerte zuweisen, Druckmessung vorbereiten */
            if(bBpFlag == enI2C_Ready){           /* Anforderung erledigt */
                dwUT  = (dword)(abBpData[0] << 8);
                dwUT |= abBpData[1];
             	x1 = (((int32)dwUT - stBpCalibrate.AC6) * stBpCalibrate.AC5) >> 15;
                x2 = ((int32)(stBpCalibrate.MC << 11)) / (x1 + stBpCalibrate.MD);
                iDeltaTemp = (x1 + x2);
                b5 = (b5 + iDeltaTemp) >> 1;      /* gleitender Mittelwert */
                i = ((b5 + 8) >> 4);              /* Temperatur * 0,1 C */ 
                iTemperature = (iTemperature + i) >> 1; 
                /* Drucksensor vorbereiten */
                stTrxRW.bType = enI2C_Write;      /* Schreibanforderung */
                stTrxRW.bSize = 1;                /* nur ein Byte mglich */
                stTrxRW.bRgnb = BP_nSensorRgnb;   /* Datenregisternummer */  
                abBpData[0] = BP_nBaroValue;      /* Schreibdaten */
                bBpFlag = enI2C_Clear;
                I2C_bTrxRequest(stTrxRW); 
                bBpState = enBpBaroRead;
                }
            break;
        case enBpBaroRead:                        /* Druck-Rohdaten anfordern */
            if(bBpFlag == enI2C_Ready){           /* Anforderung erledigt */
                stTrxRW.bType = enI2C_Read;       /* Werte auslesen */
                stTrxRW.bSize = sizeof(word);     /* 16 Bit */
                stTrxRW.bRgnb = BP_nBTValue;      /* Datenregisternummer */  
                bBpFlag = enI2C_Clear;
                I2C_bTrxRequest(stTrxRW); 
                bBpState = enBpCalculate;
                }
            break;
        case enBpCalculate:
            if(bBpFlag == enI2C_Ready){           /* Anforderung erledigt */
                dwUP  = (dword)(abBpData[0] << 8);
                dwUP |= abBpData[1];
                b6 = b5 - 4000;
                x1 = (stBpCalibrate.B2 * (b6 * b6 >> 12)) >> 11;
                x2 = stBpCalibrate.AC2 * b6 >> 11;
                x3 = x1 + x2;
                b3 = (((int32)stBpCalibrate.AC1 * 4 + x3) + 2)/4;
                x1 = stBpCalibrate.AC3 * b6 >> 13;
                x2 = (stBpCalibrate.B1 * (b6 * b6 >> 12)) >> 16;
                x3 = ((x1 + x2) + 2) >> 2;
                b4 = (stBpCalibrate.AC4 * (int32) (x3 + 32768)) >> 15;
                b7 = ((int32) dwUP - b3) * (50000);
                p = b7 < 0x80000000 ? (b7 * 2) / b4 : (b7 / b4) * 2;
                x1 = (p >> 8) * (p >> 8);
                x1 = (x1 * 3038) >> 16;
                x2 = (-7357 * p) >> 16;
                i = p + ((x1 + x2 + 3791) >> 4);
                iPressure = (iPressure + i) >> 1;
                /* Temperaturabfrage erneut starten */
                stTrxRW.bType = enI2C_Write;          /* Schreibanforderung */
                stTrxRW.bSize = 1;                    /* nur ein Byte mglich */
                stTrxRW.bRgnb = BP_nSensorRgnb;       /* Datenregisternummer */  
                abBpData[0] = BP_nTempValue;          /* Schreibdaten */
                bBpFlag = enI2C_Clear;
                I2C_bTrxRequest(stTrxRW); 
                bBpState = enBpTempRead;
                }
            break;
        case enBpError:   /* Drucksensor ist ausgefallen */
            break;
        }
    }

/***************************************************************************
* EOF: LCD.C
****************************************************************************/


