/**************************************************************************
 *                                                                        *
 *  Name:        gps.c                                                    *
 *  Description: Auswertung GPS-Daten                                     *
 *                                                                        *
 *  Date  :      09.12.2008                                               *
 *  Author:      
 *  Revision List:
  --------------------------------------------------------------------------
  Date     | Author          | Change
  --------------------------------------------------------------------------
  09.12.08 |Dr.    | Ersterstellung
  --------------------------------------------------------------------------
 **************************************************************************/

/**************************************************************************/
/*  Include file own header files                                         */
/**************************************************************************/
#include "gps.h"
#include <string.h>
//#include "maja.h"
//#include "xbee.h"
//#include "stm32f10x.h"

/**************************************************************************/
/*  Include file external header files                                    */
/**************************************************************************/
//#include <string.h>

/**************************************************************************
* Defines und Makros Zugriff                                              *
***************************************************************************/
enum{                 /* Zustnde der seriellen Empfangsemulation */
    enEmuRxWait = 0,
    enEmuRxData,
    enEmuRxStop
    };

enum{
    enGpsStart = 0,   /* Suche nach Start '$' */
    enGpsRead,        /* Daten einlesen */
    enGpsCrcHi,       /* Prfsumme */
    enGpsCrcLo
    };

#define nNoHex              0xff

#define nGpsBufferSize      128
#define nGpsBufferEntries   2
#define nGpsBufferMask      nGpsBufferEntries - 1




#define GPS_TEST

/* Struktur eines empfangenen GPS-Datensatzes */
typedef struct stGps{
    byte abData[nGpsBufferSize];
    byte bSize;
    }stGps;

const byte acGPGGA[] = "$GPGGA";
const byte acGPRMC[] = "$GPRMC";

/**************************************************************************
*    lokale Funktionsprototypen fr Funktionspointerliste                 *
***************************************************************************/



/**************************************************************************
* Makros
***************************************************************************/

/**************************************************************************
*    Variablen                                                            *
***************************************************************************/
byte bGpsRxdState; /*
* Decsription : Nu was wird wohl sein?
*/

byte abGpsRxd[nGpsBufferSize]; /*
* Description : Empfangspuffer
*/

byte bGpsRxdIdx; /*
* Description : Anzahl der GPS-Daten
*/

byte bGpsRxdCrcIntern; /*
* Description : Prfsumme 
*/

byte bGpsRxdCrcExtern; /*
* Description : Prfsumme 
*/

stGps stGpsMessages[nGpsBufferEntries]; /*
* Description : Messagebuffer
*/

byte bGpsIdxIn; /*
* Description : Zeiger auf eingehende GPS-Daten
*/

byte bGpsIdxOut; /*
* Description : Zeiger auf bearbeitete GPS-Daten 
*/

byte bGpsEmuData; /*
* Description : Emulationsvariable fr Rx-Data
*/

byte bGpsEmuMask; /*
* Description : verschiebliche Bitmaske
*/

byte bGpsEmuState; /*
* Description : Zustandsautomat fr seriellen Datenempfang
*/


struct stGpsData{ /* Struktur der GPS-Daten */ 
    byte abAltitude[nGpsStrSize];
    byte abSpeed[nGpsStrSize];
    byte abCoarse[nGpsStrSize];
    byte abUtc[nGpsStrSize];
    byte abDate[nGpsStrSize];
    byte abLatitude[nGpsStrSize];
    byte abDirLati[nGpsStrSize];
    byte abLongitude[nGpsStrSize];
    byte abDirLong[nGpsStrSize];
    byte abValid[nGpsStrSize];
    }stGpsData;

byte bGpsTimeOut; /*
* Description : bewertet Lcken bei GPS Datenstzen
*/

//const byte acTestGPRMC[] = "GPRMC,095634.316,V,5109.9119,N,1108.0903,E,0.19,33.26,060109,,,A*";
//const byte acTestGPGGA[] = "GPGGA,195634.316,5109.9119,N,01108.0903,E,1,10,1.2,27.0,M,-34.2,M,,0000,*";
const byte acTestGPRMC[] = "GPRMC,095634.316,V,0.0000,N,0.0000,E,0.00,0.00,060109,,,A*";
const byte acTestGPGGA[] = "GPGGA,195634.316,5109.9119,N,01108.0903,E,1,10,1.2,0.0,M,-34.2,M,,0000,*";
const byte abLeer[] = "----------";         /* ungltiges Argument */

/**************************************************************************
 *  lokale Funktionen
 **************************************************************************/
/* Ausrichten und Auffllen mit vorlaufenden Leerzeichen */
void vStrAlign(byte *pSource, byte *pTarget, byte bSize, byte bMax){
    byte *s, *t;
    s = pSource+bSize-1;
    t = pTarget+bMax-1;
    while(bMax){
        if(bSize) { *t-- = *s--; bSize--; }
        else      { *t-- = ' ';           }
        bMax--;
        }
    }

/* stckweises kopieren von mit ',' getrennten Stringabschnitten */
byte bStrCopy(byte *pSource, byte *pTarget){
    byte i = 0;
    if(*pSource == ','){              /* auf "Leerstring" prfen */
        *pTarget = '\0';              /* Null-Terminator in Ziel schreiben */
        i = 1;
        }
    else{
        while(i < nGpsStrSize-1){     /* berschreiben des Zielpuffers vermeiden */
            *pTarget++ = *pSource++;
            i++;
            if(*pSource == ','){      /* ',' Ende-Kennzeichen */
                *pTarget = '\0';
                i++;
                break; 
                }
            }
        }
    return i;
    } 

/* Umwandlung einer ASCII-Wertes in eine Hex-Zahl */
byte bAsciiToHex( byte bChar ){
    if((bChar >= '0') && (bChar <= '9')){
        bChar = bChar - '0';
        }
    else if((bChar >= 'A') && (bChar <= 'F')){
        bChar = bChar - '7';
        }
    else if((bChar >= 'a') && (bChar <= 'f')){
        bChar = bChar - 'W';
        }
    else bChar = nNoHex;
    return bChar;
    }


/* Vergleich zweier Strings */
bool boCompareStr( byte *p1, byte *p2 ){
    byte i = True;
    while((*p1 != '\0') || (*p2 != '\0')){
        if(*p1 != *p2){
            i = False;
            break;
            }
        p1++;
        p2++;
        }
    return i;
    }

/**************************************************************************
 *  Auslesen der ASCII-Datenstze
 **************************************************************************/
byte GPS_bDataStr( byte *p, byte bSelect ){
    byte i = 0;
    if(stGpsData.abValid[0] == nGpsValid){
        switch(bSelect){
            case enAltitude:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abAltitude[i];
                    if(stGpsData.abAltitude[i] == '\0') break;
                    }
                break;
            case enSpeed:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abSpeed[i];
                    if(stGpsData.abSpeed[i] == '\0') break;
                    }
                break;
            case enCoarse:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abCoarse[i];
                    if(stGpsData.abCoarse[i] == '\0') break;
                    }
                break;
            case enUtc:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abUtc[i];
                    if(stGpsData.abUtc[i] == '\0') break;
                    }
                break;
            case enLatitude:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abLatitude[i];
                    if(stGpsData.abLatitude[i] == '\0') break;
                    }
                break;
            case enLongitude:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abLongitude[i];
                    if(stGpsData.abLongitude[i] == '\0') break;
                    }
                break;
            case enValid:
                for(i = 0; i < nGpsStrSize; i++){
                    *p++ = stGpsData.abValid[i];
                    if(stGpsData.abValid[i] == '\0') break;
                    }
                break;
            }
        }
    return i;
    }

/**************************************************************************
 *  Initialisierung
 **************************************************************************/
void GPS_vInit( void ){
    bGpsRxdIdx = bGpsIdxIn = bGpsIdxOut = 0;
    stGpsData.abValid[0] = nGpsNotValid;
    stGpsData.abValid[1] = '\0';
    bGpsRxdState = enGpsStart;
    bGpsEmuState = enEmuRxWait;
    bGpsTimeOut = nGpsTimeOut;
    }

/**************************************************************************
 *  Zeiger auf GPS-Werte in ASCII
 **************************************************************************/
byte * GPS_vGetPointer( byte bArg ){
    byte *p;
    switch(bArg){
        case enAltitude:
            p = &stGpsData.abAltitude[0];
            break;
        case enSpeed:
            p = &stGpsData.abSpeed[0];
            break;
        case enCoarse:
            p = &stGpsData.abCoarse[0];
            break;
        case enLongitude:
            p = &stGpsData.abLongitude[0];
            break;
        case enLatitude:
            p = &stGpsData.abLatitude[0];
            break;
        default:
            p = (byte*)&abLeer[0];
            break;
        }
    return p;
    }



/**************************************************************************
 *  emulierte Rxd-Funktion (anstelle der Rxd-ISR)
 **************************************************************************/
void GPS_vIsrEmu( byte bRxd ){
    switch(bGpsRxdState){
        case enGpsStart:                                      /* Suche nach Start '$' */
            if(bRxd == '$'){
                bGpsRxdIdx = bGpsRxdCrcIntern = 0;
                bGpsRxdState = enGpsRead;
                }
            break;
        case enGpsRead:                                       /* Daten einlesen */
            if(bGpsRxdIdx < nGpsBufferSize){
                if(bRxd != '*'){
                    abGpsRxd[bGpsRxdIdx++] = bRxd;
                    bGpsRxdCrcIntern ^= bRxd;                 /* CRC mitrechnen */
                    }
                else{
                    bGpsRxdCrcExtern = 0;     
                    bGpsRxdState = enGpsCrcHi;                /* Botschaft berprfen */
                    }
                }
            else{
                bGpsRxdState = enGpsStart;                    /* berlauf */
                }
            break;
        case enGpsCrcHi:                                      /* Prfsumme einlesen */
            bRxd = bAsciiToHex(bRxd);
            if(bRxd == nNoHex){
                bGpsRxdState = enGpsStart;                    /* kein HEX-Wert */
                }
            else{
                bGpsRxdCrcExtern = bRxd << 4;                 /* Hi-Wert zuweisen */
                bGpsRxdState = enGpsCrcLo;
                }
            break;
        case enGpsCrcLo:
            bRxd = bAsciiToHex(bRxd);
            if(bRxd != nNoHex){
                bGpsRxdCrcExtern |= bRxd;                     /* Hi-Wert zuweisen */
                if(bGpsRxdCrcExtern == bGpsRxdCrcIntern){
                    bRxd = (bGpsIdxIn + 1) & nGpsBufferMask;  /* nchsten Buffer adressieren */
                    if(bRxd != bGpsIdxOut){                   /* noch Platz im Puffer ? */
                        bGpsIdxIn = bRxd;                     /* bGpsIdxIn++ */
                        for(bRxd = 0; bRxd < bGpsRxdIdx; bRxd++){
                            stGpsMessages[bGpsIdxIn].abData[bRxd] = abGpsRxd[bRxd]; 
                            }
                        stGpsMessages[bGpsIdxIn].bSize = bGpsRxdIdx;
                        }
                    }
                }
            bGpsRxdState = enGpsStart;
            break;
        }
    }

/**************************************************************************
 *  Main-Loop
 *  - zyklische Ausfhrung der Liste der Containerfunktionen
 **************************************************************************/
void GPS_vMain( cbyte bId ){
    byte i, j, *p, *t, abTmp[nGpsStrSize];
    if(bGpsIdxIn != bGpsIdxOut){
        bGpsTimeOut = nGpsTimeOut;                  /* Timeout zurcksetzen */
        bGpsIdxOut = (bGpsIdxOut + 1) & nGpsBufferMask; 
        if(strstr((const char*)stGpsMessages[bGpsIdxOut].abData,"GPRMC")){
            p  = &stGpsMessages[bGpsIdxOut].abData[6];
            //p  = (byte*)&acTestGPRMC[6]; /* <- Teststring */
            p += bStrCopy(p, stGpsData.abUtc);      /* Zeit umkopieren */
            p += bStrCopy(p, stGpsData.abValid);    /* 'A'-gltig, 'V'-ungltig */
            t = p;
            if(*(p+1) == ',') {                     /* keine Breite */
                p += bStrCopy((byte*)&acTestGPRMC[19], abTmp);
                vStrAlign(abTmp, stGpsData.abLatitude, (p-t), 10); 
                p = t; /* +1 ?? */
                }
            else{
                p += bStrCopy(p, abTmp);
                vStrAlign(abTmp, stGpsData.abLatitude, (p-t), 10); 
                }
            if(*(p/*+1*/) == ',') {                     /* kein N/S-Indikator */  
                bStrCopy((byte*)&acTestGPRMC[26], stGpsData.abDirLati); 
                p++;
                }
            else{
                p += bStrCopy(p, stGpsData.abDirLati);
                }
            /* Lnge normieren */
            t = p;
            if(*(p+1) == ','){                      /* keine Lnge */
                p += bStrCopy((byte*)&acTestGPRMC[28], abTmp);
                vStrAlign(abTmp, stGpsData.abLongitude, (p-t), 11); 
                p = t;
                }
            else{
                p += bStrCopy(p, abTmp);
                vStrAlign(abTmp, stGpsData.abLongitude, (p-t), 11); 
                }
            if(*(p/*+1*/) == ','){                      /* kein E/W-Indikator */
                bStrCopy((byte*)&acTestGPRMC[35], stGpsData.abDirLong);
                p++;
                }
            else{
                p += bStrCopy(p, stGpsData.abDirLong);
                }
            /* Geschwindigkeit aus GPS oder Default temporr laden und ausrichten  */ 
            t = p;                                  /* Position des Zeiger auf GPS-Message merken */
            if(*(p+1) == ','){                      /* "123.45" Format */
                p += bStrCopy((byte*)&acTestGPRMC[37], abTmp);
                vStrAlign(abTmp, stGpsData.abSpeed, (p-t), 7); 
                p = t;                              /* Pointer korrigieren */
                }
            else {
                p += bStrCopy(p, abTmp);
                vStrAlign(abTmp, stGpsData.abSpeed, (p-t), 7); /* auf "123.45" Stellen normieren */ 
                }
            /* Kurs ebenfalls normieren */
            t = p;
            if(*(p+1) == ','){
                p += bStrCopy((byte*)&acTestGPRMC[42], abTmp);
                vStrAlign(abTmp, stGpsData.abCoarse, (p-t), 7); 
                p = t;                              /* Pointer korrigieren */
                }
            else {
                p += bStrCopy(p, abTmp);
                vStrAlign(abTmp, stGpsData.abCoarse, (p-t), 7);  
                }
            p += bStrCopy(p, stGpsData.abDate);     /* Datum */
            }
        else if(strstr((const char*)stGpsMessages[bGpsIdxOut].abData,"GPGGA")){
            p = &stGpsMessages[bGpsIdxOut].abData[0];
            j = 0;                                  /* Anzahl der Kommata */
            for(i = 0; i < stGpsMessages[bGpsIdxOut].bSize; i++){
                if(*p++ == ',')j++;
                if(j == 9){                         /* nach 9. Komma steht die Hhe */
                    t = p;
                    if(*p == ','){                  /* keine gltige Hhe */
                        p += bStrCopy((byte*)&acTestGPGGA[51], abTmp);
                        vStrAlign(abTmp, stGpsData.abAltitude, (p-t), 8); 
                        p = t;                              /* Pointer korrigieren */
                        }
                    else{                           /* GPS hat Daten */
                        p += bStrCopy(p, abTmp);
                        vStrAlign(abTmp, stGpsData.abAltitude, (p-t), 8);  
                        }
                    break;
                    }
                }
            }
        }
    }

/* EOF */


