/**************************************************************************
 *                       
 *  Name:        i2c.c  
 *  Description: low level routinen fr Magnetometer und Accelerometer  
 *  Date  :      19.05.2010 
 *  Author:      
 *  Revision List:
  --------------------------------------------------------------------------
  Date     | Author          | Change
  --------------------------------------------------------------------------
  19.05.10 |                 | Ersterstellung
  --------------------------------------------------------------------------
           |                 |
  --------------------------------------------------------------------------
 **************************************************************************/

/***************************************************************************
  HEADER-FILES (Only those that are needed in this file)
****************************************************************************/

#include "i2c.h"
//#include "maja.h"

/* Foreign headerfiles */


#define mcI2CWait(x)  {wDelay = x; while(wDelay--);}
#define n600ns      1

/***************************************************************************
*    Variablen
****************************************************************************/
volatile word wDelay; /*
* Description : Laufzeitzhler
*/

I2C_stTrx stSingleTrxList[nSingleListSize]; /*
* Description : Ringpuffer fr Einzelkommunikationsauftrge
*/

byte bSingleIdxNew; /*
* Description : Index auf eingeschriebene Auftrge
*/

byte bSingleIdxRun; /*
* Description : Index auf laufende Auftrge 
*/

byte b200msTimer; /*
 * Description : Zhler fr 200ms Tick
 */
byte abMag[8];
int16 iMx;
int16 iMy;
int16 iMz;

//byte abWriteI2C[10];

/***************************************************************************
*    Funktion: I2C-Schreib-Lese-Routine (Master)
*    Autor:    
****************************************************************************/
byte I2CCtrl( byte bCtrl, byte bData, byte bAck ){
    byte bBit = 0x80;
    switch( bCtrl ){
        case enI2C_Start:                     /* start condition */
            mcSDA_Out();
            mcSDA_Hi(); mcSCL_Hi();           /* Data, Clk high */
            mcSDA_Lo();                       /* SDA low (start) */
            mcI2CWait(n600ns)                 /* th(ST) */
            mcSCL_Lo();
            break;
        case enI2C_Write:                     /* write + ack */
            mcSDA_Out();                      /* data output */
            while( bBit ){
                if(bData & bBit) mcSDA_Hi();  /* set SDA */
                else             mcSDA_Lo();  /* res SDA */
                mcI2CWait(n600ns);            /* tsu(SDA) */
                mcSCL_Hi();                   /* SCL high */
                mcI2CWait(n600ns);            /* tw(SCLh) */
                mcSCL_Lo();
                bBit >>= 1;                   /* next bit */
                }
            mcSDA_In();                       /* data input */
            mcSCL_Hi();                       /* SCL high */
            mcI2CWait(n600ns);                /* tw(SCLh) */
            bData = !boSDA();                 /* get ack (inverted) */
            mcSCL_Lo();
            mcI2CWait(n600ns*10);             /* slave delay */
            break;
        case enI2C_Read:                      /* read + ack */
            mcSDA_In();                       /* data input */
            while( bBit ){
                mcSCL_Hi();                   /* SCL high */
                mcI2CWait(n600ns); 
                if(boSDA()) bData |= bBit;
                else        bData &=~bBit;
                mcSCL_Lo();                   /* SCL low */
                mcI2CWait(n600ns);   
                bBit >>= 1;                   /* next bit */
                }
            mcSDA_Out();                      /* data output */
            if(bAck) mcSDA_Lo();              /* set ack (inverted) */
            else     mcSDA_Hi();
            mcI2CWait(n600ns); 
            mcSCL_Hi();                       /* SCL high */
            mcI2CWait(n600ns);
            mcSCL_Lo();                       /* SCL low */
            mcI2CWait(n600ns);
            mcSDA_Lo();
            mcI2CWait(n600ns*10);             /* slave delay */
            break;
        case enI2C_Stop:                      /* stop condition */
            mcSDA_Out();                      /* data output */
            mcSCL_Hi();                       /* SCL high */
            mcI2CWait(n600ns);
            mcSDA_Hi();                       /* SDA high (stop) */
            break;
        }
    return bData;                             /* return data|ack */
    }


/***************************************************************************
*    Funktion: Daten vom I2C-Gert lesen
*    Autor:    
****************************************************************************/
void vReadData( I2C_stTrx stRx ){ 
    byte i;
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, (stRx.bAddr&0xfe), 1); /* Adresse schreiben */
    I2CCtrl(enI2C_Write, stRx.bRgnb, 1);        /* welches Register */
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, (stRx.bAddr|0x01), 1); /* Lesebefehl */
    for(i = 0; i < stRx.bSize-1; i++){          /* Daten auslesen */
        *stRx.pData++ = I2CCtrl(enI2C_Read, 0, 1);
        }
    *stRx.pData = I2CCtrl(enI2C_Read, 0, 0);
    I2CCtrl(enI2C_Stop, 0, 0);
    *stRx.pFlag = enI2C_Ready;
    }

/***************************************************************************
*    Funktion: Daten zum I2C-Gert schreiben
*    Autor:    
****************************************************************************/
void vWriteData( I2C_stTrx stRx ){ 
    //byte i;
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, (stRx.bAddr&0xfe), 1); /* Adresse schreiben */
    I2CCtrl(enI2C_Write, stRx.bRgnb, 1);        /* welches Register */
    I2CCtrl(enI2C_Write, *stRx.pData, 1);       /* Datum auf Register schreiben */
    I2CCtrl(enI2C_Stop, 0, 0);
    *stRx.pFlag = enI2C_Ready;
    }



/***************************************************************************
*    Funktion: Kommunikation anfordern 
*    Autor:    
****************************************************************************/
byte I2C_bTrxRequest( I2C_stTrx stRx ){
    byte i, bRet = False;
    if(stRx.bCycl == 0){                        /* einmaliger Schreib/Lese-Zugriff */
        i = (bSingleIdxNew+1) & nSingleListMask;
        if(i != bSingleIdxRun){                 /* berschneiden der Idx verhindern */ 
            stSingleTrxList[i] = stRx;          /* neuen Request eintragen */
            bSingleIdxNew = i;
            }
        else{
            bRet = False;                       /* Puffer voll */
            }
        }
    else{
        bRet = False;
        }
    return bRet;
    }

/***************************************************************************
*    Funktion: Initialisierung
*    Autor:    
****************************************************************************/
void I2C_vInit( void ){
    mcSCL_Hi(); 
    mcSDA_Hi(); 
    /* CRA_REG_M - Mode einstellen */
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, 0x3c, 1);
    I2CCtrl(enI2C_Write, 0x00, 1);
    I2CCtrl(enI2C_Write, 0x10, 1);              
    I2CCtrl(enI2C_Stop, 0, 0);
    /* CRB_REG_M - Empfindlichkeit einstellen */
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, 0x3c, 1);
    I2CCtrl(enI2C_Write, 0x01, 1);
    I2CCtrl(enI2C_Write, 0x20, 1);              /* 0x02, 0x40 */
    I2CCtrl(enI2C_Stop, 0, 0);
    /* Magnetsensor aufwecken */
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, 0x3c, 1);
    I2CCtrl(enI2C_Write, 0x02, 1);
    I2CCtrl(enI2C_Write, 0x01, 1);              /* aufwecken */
    I2CCtrl(enI2C_Stop, 0, 0);
    /* Beschleunigungssensor einschalten */
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, 0x30, 1);
    I2CCtrl(enI2C_Write, 0x20, 1);
    I2CCtrl(enI2C_Write, 0x27, 1);              /* normal mode */
    I2CCtrl(enI2C_Stop, 0, 0);
    I2CCtrl(enI2C_Start, 0, 0);
    I2CCtrl(enI2C_Write, 0x30, 1);
    I2CCtrl(enI2C_Write, 0x23, 1);
    I2CCtrl(enI2C_Write, 0xb0, 1);              /* BDU */
    I2CCtrl(enI2C_Stop, 0, 0);
    bSingleIdxNew = bSingleIdxRun = 0;
    b200msTimer = 0;
    }

/***************************************************************************
*    Funktion: I2C-Abfragen koordinieren
*    Autor:    
****************************************************************************/
void I2C_vMain( cbyte cID ){
    byte i;
    //int16 iAvg;
#if 0 /* alle Abfragen totlegen */
    if(bSingleIdxNew != bSingleIdxRun){         /* Anforderungen ausfhren */
        bSingleIdxRun = (bSingleIdxRun+1) & nSingleListMask;
        if(stSingleTrxList[bSingleIdxRun].bType == enI2C_Read){ 
            vReadData(stSingleTrxList[bSingleIdxRun]);
            }
        else if(stSingleTrxList[bSingleIdxRun].bType == enI2C_Write){ 
             vWriteData(stSingleTrxList[bSingleIdxRun]);
            }
        }
#endif
    b200msTimer++;
    i = 0;
    if(b200msTimer == 9){ /* Magnetometer-Statusregister lesen */
        b200msTimer = 0;
        I2CCtrl(enI2C_Start, 0, 0);
        I2CCtrl(enI2C_Write, 0x3c, 1);  /* Adresse schreiben */
        I2CCtrl(enI2C_Write, 0x09, 1);  /* welches Register */
        I2CCtrl(enI2C_Start, 0, 0);
        I2CCtrl(enI2C_Write, 0x3d, 1);  /* Lesebefehl */
        i = I2CCtrl(enI2C_Read, 0, 0);
        I2CCtrl(enI2C_Stop, 0, 0);
        }
    if(i & 0x01){         /* Magnetometer-Daten sind verfgbar */
        //mcLed2On();
        //mcLed2Toggle();
        I2CCtrl(enI2C_Start, 0, 0);
        I2CCtrl(enI2C_Write, 0x3c, 1);
        I2CCtrl(enI2C_Write, 0x03|0x80, 1);
        I2CCtrl(enI2C_Start, 0, 0);
        I2CCtrl(enI2C_Write, 0x3d, 1);
        for(i = 0; i < 5; i++){
            abMag[i] = I2CCtrl(enI2C_Read, 0, 1);
            }
        abMag[i] = I2CCtrl(enI2C_Read, 0, 0);
        I2CCtrl(enI2C_Stop, 0, 0);
#if 0
        iAvg  = (int16)(abMag[0] << 8);
        iAvg |= abMag[1];
        iMx   = (iMx + iAvg) >> 1;
        iAvg  = (int16)(abMag[2] << 8);
        iAvg |= abMag[3];
        iMy   = (iMy + iAvg) >> 1;
        iAvg  = (int16)(abMag[4] << 8);
        iAvg |= abMag[5];
        iMz   = (iMz + iAvg) >> 1;
#else
        iMx  = (int16)(abMag[0] << 8);
        iMx |= abMag[1];
        iMy  = (int16)(abMag[2] << 8);
        iMy |= abMag[3];
        iMz  = (int16)(abMag[4] << 8);
        iMz |= abMag[5];
#endif
#if 1
        /* Magnetsensor aufwecken */
        I2CCtrl(enI2C_Start, 0, 0);
        I2CCtrl(enI2C_Write, 0x3c, 1);
        I2CCtrl(enI2C_Write, 0x02, 1);
        I2CCtrl(enI2C_Write, 0x01, 1);              /* aufwecken */
        I2CCtrl(enI2C_Stop, 0, 0);        //mcLed2Off();
#endif
        }
    }

/***************************************************************************
* EOF: LCD.C
****************************************************************************/


